import Ogma from '@linkurious/ogma';
import React, { useEffect, useRef, useContext } from 'react';
import { Context as GraphContext } from './GraphContext.jsx';
import { selectNode, hoverNode } from './actions.js';

export default function OgmaWrapper() {
  const containerId = 'ogma-container';
  // initialize ogma and get a reference to it
  const ogmaRef = useRef(null);

  const {
    state: { graph },
    dispatch
  } = useContext(GraphContext);

  useEffect(() => {
    ogmaRef.current = new Ogma({ container: containerId });
  }, []);

  useEffect(() => {
    const onNodeClick = id => dispatch(selectNode(id));
    // as suggested by react debug tools, due to ref lifecycle
    const ogma = ogmaRef.current;
    const clickHandler = ({ target }) => {
      if (target && target.isNode) return onNodeClick(target.getId());
      onNodeClick(null);
    };

    const hoverHandler = ({ target }) => {
      const hoveredNode = target && target.isNode ? target.getId() : null;
      requestAnimationFrame(() => dispatch(hoverNode(hoveredNode)));
    };

    ogma.events.on('click', clickHandler);
    ogma.events.on('mouseover', hoverHandler);

    return () => {
      ogma.events.off(clickHandler);
      ogma.events.off(hoverHandler);
    };
  }, [dispatch]);

  // update graph
  useEffect(() => {
    const ogma = ogmaRef.current;
    ogma.clearGraph();
    ogma
      .setGraph(graph)
      .then(() => ogma.layouts.force({ locate: graph.nodes.length > 0 }))
      .then(() => ogma.view.locateGraph())
      .then(() => {
        // you can update the graph state here
        // const nodeSettings = { attributes: ["x", "y", "radius"] };
        // const edgeSettings = { attributes: ["width"] };
        // graph.nodes = ogmaRef.current.getNodes().toJSON(nodeSettings);
        // graph.edges = ogmaRef.current.getEdges().toJSON(edgeSettings);
      });
  }, [graph]);

  return (
    <div className="OgmaWrapper">
      <div id={containerId} />
    </div>
  );
}
